library fly_animation;

import 'package:flutter/material.dart';

///FlyAnimationWidget to animate widgets from one position to another
///Add FlyAnimationWidget as root of your widget tree
// ignore: must_be_immutable
class FlyAnimationWidget extends StatefulWidget {
  ///key of the widget where animation ends
  final GlobalKey animationEndKey;
  ///number of animation widgets need to be created
  int countOfAnimationWidgets;
  ///top position of the animation widget to display on screen
  final double topPosition;
  ///widgets tree to be displayed in screen
  final Widget childWidget;
  ///animation widget
  final Widget animationWidget;
  ///animation widget width
  final int animationWidgetWidth;
  ///animation widget height
  final int animationWidgetHeight;
  ///animation duration in millis
  final int animationDurationInMillis;
  ///pass true if you want to hide animation widget on animation complete
  final bool hideAnimationWidgetOnComplete;

  FlyAnimationWidget(
      {required this.animationEndKey,
      required this.countOfAnimationWidgets,
      required this.topPosition,
      required this.childWidget,
      required this.animationWidget,
      required this.animationWidgetWidth,
      required this.animationWidgetHeight,
      required this.animationDurationInMillis,
      required this.hideAnimationWidgetOnComplete})
      : assert(countOfAnimationWidgets > 0,
            'Count of animation widget to be shown must be greater than 0'),
        assert(topPosition >= 0,
            'Animation widget\'s top position must be greater than or equal to zero'),
        assert(animationWidgetWidth >= 0,
            'Animation widget\'s width must be greater than or equal to zero & it must be greater than animationWidget\'s width'),
        assert(animationWidgetHeight >= 0,
            'Animation widget\'s height must be greater than or equal to zero & it must be greater than animationWidget\'s height'),
        assert(animationDurationInMillis >= 0,
            'Animation duration must be greater than or equal to zero');

  @override
  _FlyAnimationWidgetState createState() => _FlyAnimationWidgetState();
}

class _FlyAnimationWidgetState extends State<FlyAnimationWidget> {
  ///temp key of the top widget in the Stack
  var tempKey = GlobalKey();
  ///key of the animation widget
  var animationWidgetKey = GlobalKey();
  ///X position of the widget where animation completes
  var animationEndPositionX;
  ///Y position of the widget where animation completes
  var animationEndPositionY;
  ///bool list for animation to be started
  List animationStartList = <bool>[];
  ///list to check which widget's visibility needs to be changed
  List animationVisibilityList = <bool>[];
  ///animation counter
  var animationCount = 0;
  ///check whether animation started by gesture
  var gestureDetected = false;
  ///total height of the root widget
  double stackHeight = 0.0;

  ///called when state of the object is created
  @override
  void initState() {
    _initAnimationData();
    WidgetsBinding.instance?.addPostFrameCallback(_getPosition);
    super.initState();
  }

  ///builds widgets tree
  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
      child: Stack(
        children: [
          Container(
            key: tempKey,
          ),
          widget.childWidget,
          stackHeight == 0.0
              ? Container()
              : Container(
                  height: stackHeight,
                  child: Stack(
                      key: animationWidgetKey,
                      children: _getAnimationWidgetList()),
                ),
        ],
      ),
    );
  }

  ///counts Y position of the last row of animation widgets and position of the widget where animation completes
  void _getPosition(_) {
    var screenWidth = MediaQuery.of(context).size.width;
    var positionX =

        (MediaQuery.of(context).size.width / 2) - widget.animationWidgetWidth;
    var positionY = widget.topPosition;
    var count = 0;
    for (int i = 0; i < widget.countOfAnimationWidgets; i++) {
      if (i % 2 == 0) {
        positionX = positionX - widget.animationWidgetWidth * count;
        count++;
      } else {
        positionX = positionX + widget.animationWidgetWidth * count;
        count++;
      }
      if (positionX < 16 ||
          positionX > (screenWidth - (widget.animationWidgetWidth + 16))) {
        positionX = (MediaQuery.of(context).size.width / 2) -
            widget.animationWidgetWidth;
        positionY = positionY + widget.animationWidgetHeight;
        count = 1;
      }
    }

    var renderBox1 =
        widget.animationEndKey.currentContext?.findRenderObject() as RenderBox;
    final position1 = renderBox1.localToGlobal(Offset.zero);
    animationEndPositionY = position1.dy;
    animationEndPositionX = position1.dx;

    setState(() {
      stackHeight = positionY + 90.0;
    });
  }

  ///initializes animation calculations list
  void _initAnimationData() {
    animationStartList.clear();
    animationVisibilityList.clear();
    for (var i = 0; i < widget.countOfAnimationWidgets; i++) {
      animationStartList.add(false);
      animationVisibilityList.add(true);
    }
  }

  ///counts X and Y position of the animation widgets and draws at that position,
  ///on touch of the animation widgets it starts animating that widget one by one,
  ///returns list of animation widgets
  List<Widget> _getAnimationWidgetList() {
    List<AnimatedPositioned> listings = <AnimatedPositioned>[];

    var screenWidth = MediaQuery.of(context).size.width;
    var positionX =
        (MediaQuery.of(context).size.width / 2) - widget.animationWidgetWidth;
    var positionY = widget.topPosition;
    var count = 0;

    for (int i = 0; i < widget.countOfAnimationWidgets; i++) {
      if (i % 2 == 0) {
        positionX = positionX - widget.animationWidgetWidth * count;
        count++;
      } else {
        positionX = positionX + widget.animationWidgetWidth * count;
        count++;
      }
      if (positionX < 16 ||
          positionX > (screenWidth - (widget.animationWidgetWidth + 16))) {
        positionX = (MediaQuery.of(context).size.width / 2) -
            widget.animationWidgetWidth;
        positionY = positionY + widget.animationWidgetHeight;
        count = 1;
      }

      listings.add(
        AnimatedPositioned(
          top: animationStartList[i] ? animationEndPositionY : positionY,
          left: animationStartList[i] ? animationEndPositionX : positionX,
          duration: Duration(milliseconds: widget.animationDurationInMillis),
          onEnd: () {
            if (animationCount < widget.countOfAnimationWidgets - 1) {
              setState(() {
                if (widget.hideAnimationWidgetOnComplete) {
                  animationVisibilityList[animationCount] =
                      !animationVisibilityList[animationCount];
                }
                animationCount++;
                animationStartList[animationCount] =
                    !animationStartList[animationCount];
                // totalStars++;
              });
            } else if (animationCount == widget.countOfAnimationWidgets - 1) {
              setState(() {
                if (widget.hideAnimationWidgetOnComplete) {
                  animationVisibilityList[animationCount] =
                      !animationVisibilityList[animationCount];

                  widget.countOfAnimationWidgets = 0;
                }
                // totalStars++;
              });
            }
          },
          curve: Curves.easeInOut,
          child: InkWell(
            onTap: () async {
              if (!gestureDetected) {
                var renderBoxTemp =
                    tempKey.currentContext?.findRenderObject() as RenderBox;
                final positionTemp = renderBoxTemp.localToGlobal(Offset.zero);

                var renderBox = widget.animationEndKey.currentContext
                    ?.findRenderObject() as RenderBox;
                final position = renderBox.localToGlobal(Offset.zero);

                animationEndPositionY = position.dy - positionTemp.dy;
                animationEndPositionX = position.dx;

                if (animationCount < widget.countOfAnimationWidgets) {
                  setState(() {
                    gestureDetected = true;
                    animationStartList[animationCount] =
                        !animationStartList[animationCount];
                  });
                }
              }
            },
            child: Visibility(
                visible: widget.hideAnimationWidgetOnComplete
                    ? animationVisibilityList[i]
                    : true,
                child: widget.animationWidget),
          ),
        ),
      );
    }
    return listings;
  }
}
